/* PerforceVcsProvider.java $Revision:$  $Date:$
 *
 * Copyright (C) 2001  Mike Swainston-Rainford
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.togethersoft.modules.p4vcs;

import com.perforce.api.Change;
import com.perforce.api.Client;
import com.perforce.api.CommitException;
import com.perforce.api.Debug;
import com.perforce.api.Env;
import com.perforce.api.FileEntry;
import com.perforce.api.PerforceException;
import com.perforce.api.SubmitException;
import com.perforce.api.User;
import com.perforce.api.Utils;
import com.togethersoft.modules.p4vcs.dialogs.DiffOptionsDialog;
import com.togethersoft.modules.p4vcs.dialogs.AdvancedGetOptionsDialog;
import com.togethersoft.modules.p4vcs.dialogs.AdvancedOptionsDialog;
import com.togethersoft.modules.p4vcs.dialogs.FilePropertyDialog;
import com.togethersoft.modules.p4vcs.dialogs.FileHistoryDialog;
import com.togethersoft.modules.p4vcs.dialogs.SelectPasswordDialog;
import com.togethersoft.modules.p4vcs.dialogs.SelectProjectDialog;
import com.togethersoft.openapi.ide.IdeAccess;
import com.togethersoft.openapi.ide.message.IdeMessageManagerAccess;
import com.togethersoft.openapi.ide.message.IdeMessagePage;
import com.togethersoft.openapi.ide.message.IdeMessageType;
import com.togethersoft.openapi.ide.window.IdeWindowManagerAccess;
import com.togethersoft.openapi.vcs.CapabilityException;
import com.togethersoft.openapi.vcs.DefaultVcsProvider;
import com.togethersoft.openapi.vcs.VcsCommitInfo;
import com.togethersoft.openapi.vcs.VcsFileInfo;
import com.togethersoft.openapi.vcs.VcsProviderException;
import com.togethersoft.openapi.vcs.VcsProviderHelper;
import com.togethersoft.openapi.vcs.VcsProviderListener;
import com.togethersoft.openapi.vcs.VcsUpdateInfo;
import com.togethersoft.openapi.vcs.VcsVersionInfo;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Properties;
import java.util.Vector;
import javax.swing.JDialog;
import javax.swing.SwingUtilities;
import com.togethersoft.modules.p4vcs.dialogs.FileEntryTableModel;
import javax.swing.table.TableModel;

/**
 * This class provides the VCS client functionality for Perforce P4
 * @author <a href="mailto:mike@mjs-r.com">Mike Swainston-Rainford</a>
 * @version $Date:$ $Revision:$
 */
public class PerforceVcsProvider extends DefaultVcsProvider
{
   public PerforceVcsProvider()
   {
      messagepage = IdeMessageManagerAccess.getMessageManager().openPage("Version Control");
      fentCache = new Hashtable();
      try
      {
         myHost = InetAddress.getLocalHost().getHostName();
      }
      catch (UnknownHostException uhe)
      {
         myHost = "";
      }
      if (messagepage != null)
      {
         messagepage.setTitle("Version Control");
         if (debugOn)
         {
            messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: PerforceVcsProvider constructor");
         }
      }
   }

   /** Returns "P4" as provider ID. */
   public String getID()
   {
      //    if (debugOn) {
      //      messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: getID called");
      //    }
      return "P4";
   }

   /**
    * Starts new session. A session is defined by the root working directory and associated version
    * control system's project. VcsProvider interface assumes all files in root
    * working directory and all its subdirectories may be put udner version
    * control in root project and its subprojects. After session has started,
    * current working directory is assumed to be the root one.
    * @param rootProject VCS project to be associated with root working directory
    * @param rooWorkingDir root working directory
    * @param user - login user name
    * @param password - users's password to login
    */
   public void startSession(String rootProject, File rootWorkingDir, String user, String password) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION,
             "<Perforce Provider>: startSession called with password and user");
      }
      if (rootProject == null || "".equals(rootProject) || rootWorkingDir == null)
      {
         throw new IllegalArgumentException("Null argument");
      }
      messagepage.printMessage(IdeMessageType.INFORMATION,
          "<Perforce Provider>: startSession called with rootProject " + rootProject);

      Properties myProperties = PerforceVcsProviderType.getProperties();
      if (!rootProject.startsWith("<Perforce Project Root>"))
      {
         myProperties.setProperty("p4.client", rootProject);
      }
      selectPassword(myProperties);

      p4Env = new Env(myProperties);
      Debug.setProperties(myProperties);
      User myUser = User.getUser(getEnv(), p4Env.getUser());
      setP4UserEmail(myUser.getEmail());

      String dbg = myProperties.getProperty("p4.debug");
      if (null != dbg && dbg.equals("true"))
      {
         debugOn = true;
      }
      else
      {
         debugOn = false;
      }
      super.startSession(rootProject, rootWorkingDir, p4Env.getUser(), p4Env.getPassword());
      messagepage.printMessage(IdeMessageType.INFORMATION,
          "<Perforce Provider>: P4User:" + p4Env.getUser() + " P4Client:" + p4Env.getClient() + " P4Port:" + p4Env.getPort());
   }

   /**
    * Ends current session. After this method called, current working directory and root working
    * directory, as well as current and root projects are undefined.
    * It's possiblie to execute only commands not requiring running session.
    */
   public void endSession() throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: endSession called");
      }
      super.endSession();
      p4Env = null;
   }

   /** Returns true if this VcsProvider instance is valid and can accept calls to any methods. */
   public boolean isValid()
   {
      //    if (debugOn) {
      //      messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: isValid called");
      //    }
      //return VcsProviderRegistryAccess.getRegistry().getProviderType(getID()).isProviderValid(this);
      return VcsProviderHelper.getInstance().isProviderRegistered(this);
   }

   /**
    * Called by the user of VcsProvider when this instance is not used anymore.
    * Implementation should free all allocated objects and be prepared for garbage collector.
    */
   public void free()
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: free called");
      }
      super.free();
      Utils.cleanUp(); // lose all cached objects
      p4Env = null;
   }

   /** Returns true if there is currently a session opened, false otherwise */
   public boolean isSessionStarted()
   {
      //    if (debugOn) {
      //      messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: isSessionStarted called");
      //    }
      return (getRootProject() != null && getRootWorkingDirectory() != null);
   }

   /**
    *  Used to query status information about a list of selected files. This function
    * must provide fast retrieving of files status, possibly cache the status within a session.
    *  @param name of file in current working directory
    *  @return status of file: collection of bitflags from FILE_STATUS_*
    */
   public int getFileStatus(String filename) throws VcsProviderException
   {
      if (debugOn)
      {
         //messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: getFileStatus called on - "+filename);
      }
      FileEntry fileEntry = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + filename);
      return getFileStatus(fileEntry);
   }

   public int getFileStatus(FileEntry fileEntry)
   {
      int status = 0;
      if (fileEntry.getHeadRev() < 1)
      {
         status = FILE_STATUS_UNKNOWN;
      }
      if (fileEntry.getHeadRev() > 0 && ! "edit".equals(fileEntry.getAction()) && ! "delete".equals(fileEntry.getAction()))
      {
         status = FILE_STATUS_CHECKEDIN;
      }
      if (fileEntry.getHeadRev() > 0 && ("edit".equals(fileEntry.getAction()) || "delete".equals(fileEntry.getAction())))
      {
         status = FILE_STATUS_CHECKEDOUT;
      }
      if (fileEntry.getHeadRev() > fileEntry.getHaveRev())
      {
         status = status + FILE_STATUS_NEEDS_UPDATE;
      }
      if (fileEntry.isModifiedLocally())
         status = status + FILE_STATUS_MODIFIED;
      if (fileEntry.isOtherLock())
      {
         //if set to locked some vcs operations are disabled
         //so ignore this status - not relevant to p4?
         // or make it a global option that user can set?
         status = status + FILE_STATUS_LOCKED;
      }
      if (fileEntry.isUnresolved())
      {
         status = status + FILE_STATUS_CONFLICT;
      }
      return status;
   }

   /**
    * Displays a dialog which allows the user to select a project in repository.
    * This command can be executed with session not started. If a session is already started, it may be closed.
    * @param defaultProject project to be used as default. If null assume none is specified
    * @param workingDir the working directory assumed to be associated with the project selected by the user
    * @param createNewProject if true, allow the user to create new project.
    * @param selectedProject a StringBuffer to return user selected project in
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execSelectProject(String defaultProject, File workingDir, boolean createNewProject, StringBuffer selectedProject)
       throws VcsProviderException
       {
          if (debugOn)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execSelectProject called");
          }
          Properties myProperties = PerforceVcsProviderType.getProperties();
          selectPassword(myProperties);

          Env projectEnv = new Env(myProperties);
          Debug.setProperties(myProperties);
          Client.loadClients(projectEnv);
          Enumeration allClients = Client.getClients(projectEnv);
          ArrayList myClients = new ArrayList();
          while (allClients.hasMoreElements())
          {
             Client nextClient = (Client)allClients.nextElement();
             nextClient.sync();
             if (myHost.equals(nextClient.getHost()))
             {
                myClients.add(nextClient);
             }
          }
          Object[] clientArray = myClients.toArray();
          SelectProjectDialog selectProjectDialog = new SelectProjectDialog(clientArray);
          selectProjectDialog.showDialog();
          if (selectProjectDialog.isOK())
          {
             String clientName = selectProjectDialog.getClientName();
             selectedProject.append(clientName);
          }
          return EXEC_RESULT_OK;
   }


   private void selectPassword(Properties myProperties)
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: selectPassword called");
      }
      if (null == myProperties.getProperty("p4.password") || "".equals(myProperties.getProperty("p4.password")))
      {
         String user = myProperties.getProperty("p4.user");
         SelectPasswordDialog selectPasswordDialog = new SelectPasswordDialog(user);
         selectPasswordDialog.showDialog();
         if (selectPasswordDialog.isOK())
         {
            myProperties.setProperty("p4.user", selectPasswordDialog.getUser());
            myProperties.setProperty("p4.password", selectPasswordDialog.getPassword());
         }
      }
   }

   /**
    * Retrieves the list of file names in current VCS project missing in current
    * working directory, that is  which are to be downloaded from VCS on next
    * get project operation. A session must be started prior to call this method.
    * @param files java.util.Vector instance to be filled up with file names missing in working directory
    * @param fileName name of file in the current working directory
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execGetMissingFiles(Vector files) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execGetMissingFiles called");
      }
      throw new CapabilityException(CMD_GET_MISSING_FILES);
   }

   /**
    * Retrieves the statuses of files from current working directory with names
    * specified in <CODE>files</CODE> array. Returns result in
    * <CODE>statuses</CODE> array. A session must be started prior to call this method.
    * @param files array of file names in current working directory
    * @param statuses array of int to be filled with statuses
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execGetFilesStatuses(String[] files, int[] statuses) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execGetFilesStatuses called");
      }
      for (int i = 0; i < files.length; i++)
      {
         statuses[i] = getFileStatus(files[i]);
      }
      return EXEC_RESULT_OK;
   }

   /**
    * Retrieves the information about files Information is returned as array list of VcsFileInfo objects
    * A session must be started prior to call this method.
    * @param files array of file names in working directory; if 'wholeProject'
    * is true this parameter may be null and thus should be ignored
    * @param wholeProject if true, act on all contents of the project; 'files' parameter should be ignored in this case
    * @param projectRecursively if true, proceed the project recursively
    * @param fileInfo array list with project files information
    * @return result of command execution from EXEC_RESULT_*
    * @stereotype new
    */
   public int execGetFileInfo(String[] files, boolean wholeProject, boolean projectRecursively, List fileInfo)
       throws VcsProviderException
       {
          //if (debugOn) {
          //   messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execGetFileInfo called");
          //}
          //do directory
          if (wholeProject)
          {
             String dir = "";
             try
             {
                dir = FileEntry.convertToDepotPath(getEnv(), getCurrentWorkingDirectory().getAbsolutePath());
                if (projectRecursively)
                   dir = dir + "/...";
                else
                   dir = dir + "/*";
                //To Do: get file info on everything in the project?
             }
             catch (Exception e)
             {
                if (debugOn)
                {
                   messagepage.printMessage(IdeMessageType.ERROR,
                       "<Perforce Provider>: execGetFileInfo failed with exception \n" + e.toString());
                }
                return EXEC_RESULT_ERROR;
             }
             return EXEC_RESULT_OK;
          }
          try
          {
             for (int i = 0; i < files.length; i++)
             {
                FileEntry fent = FileEntry.getFile(getEnv(), files[i]);
                int status = getFileStatus(fent);
                VcsFileInfo myFileInfo = new P4VcsFileInfo(fent, status);
                fileInfo.add(i, myFileInfo);
             }
          }
          catch (Exception e)
          {
             if (debugOn)
             {
                messagepage.printMessage(IdeMessageType.ERROR,
                    "<Perforce Provider>: execGetFileInfo failed with exception \n" + e.toString());
             }
             return EXEC_RESULT_ERROR;
          }
          return EXEC_RESULT_OK;
   }

   /**
    * Invokes a dialog specific to souce control system with detailed properties for a file.
    * A session must be started prior to call this method.
    * @param fileName name of file in the current working directory
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execShowFileProperties(String fileName) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execShowFileProperties called");
      }
      try
      {
         FileEntry fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + fileName);
         JDialog myDialog = IdeWindowManagerAccess.getWindowManager().createDialog(null);
         FilePropertyDialog filepropDialog = new FilePropertyDialog(myDialog, true, fent, this);
         Runner myRunner = new Runner(filepropDialog);
         SwingUtilities.invokeLater(myRunner);
      }
      catch (Exception e)
      {
         if (debugOn)
         {
            messagepage.printMessage(IdeMessageType.ERROR,
                "<Perforce Provider>: execShowFileProperties failed with exception \n" + e.toString());
         }
         return EXEC_RESULT_ERROR;
      }
      return EXEC_RESULT_OK;
   }

   /**
    *  Invokes a dialog which shows history of changes to specified files or whole project.
    * A session must be started prior to call this method.
    *  @param files array of file names in working directory; if 'wholeProject'
    * is true this parameter may be null and thus should be ignored
    *  @param wholeProject if true, act on all contents of the project; 'files' parameter should be ignored in this case
    *  @param projectRecursively if true, proceed the project recursively
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execShowFilesHistory(String[] files, boolean wholeProject, boolean projectRecursively)
       throws VcsProviderException
       {
          if (debugOn)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execShowFilesHistory called");
          }
          //To Do - history for whole project
          try
          {
             for (int i = 0; i < files.length; i++)
             {
                String fileName = getCurrentWorkingDirectory() + fs + files[i];
                JDialog myDialog = IdeWindowManagerAccess.getWindowManager().createDialog(null);
                FileHistoryDialog historyDialog = new FileHistoryDialog(myDialog, true, fileName, getEnv());
                Runner myRunner = new Runner(historyDialog);
                SwingUtilities.invokeLater(myRunner);
             }
          }
          catch (Exception e)
          {
             messagepage.printMessage(IdeMessageType.ERROR,
                 "<Perforce Provider>: execShowFilesHistory failed with exception\n" + e.toString());
             return EXEC_RESULT_ERROR;
          }
          return EXEC_RESULT_OK;
   }

   /**
    * Returns the result of getting history of changes to specified file A session must be started prior to call this method.
    * @param file name of the file in working directory
    * @param versions array to receive list of file versions (0 is root version)
    * @return result of command execution from EXEC_RESULT_*
    * @stereotype new
    */
   public int execGetFileHistory(String file, List versions) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execGetFileHistory called");
      }
      throw new CapabilityException(CMD_GET_FILE_HISTORY);
   }

   /**
    * Search for versions of the file containing specified pattern A session must be started prior to call this method.
    * @param file name of the file in working directory
    * @param string pattern to be searched
    * @param searchResults array of VcsVersionInfo objects with versions which matched search criteria
    * @return result of command execution from EXEC_RESULT_*
    * @stereotype new
    */
   public int execSearchFileHistory(String file, String pattern, List searchResult) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execSearchFileHistory called");
      }
      throw new CapabilityException(CMD_SEARCH_HISTORY);
   }

   /**
    *  Returns the raw result of getting history of changes to specified files or whole project.
    * A session must be started prior to call this method.
    *  @param files array of file names in working directory; if 'wholeProject'
    * is true this parameter may be null and thus should be ignored
    *  @param wholeProject if true, act on all contents of the project; 'files' parameter should be ignored in this case
    *  @param projectRecursively if true, proceed the project recursively
    *  @param buffer a StringBuffer instance to return history data in
    *  @return result of command execution from EXEC_RESULT_*
    */
   public int execGetRawFilesHistory(String[] files, boolean wholeProject, boolean projectRecursively, StringBuffer buffer)
       throws VcsProviderException
       {
          if (debugOn)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execGetRawFilesHistory called");
          }
          throw new CapabilityException(CMD_GET_RAW_FILES_HISTORY);
   }

   /**
    * Invokes file difference dialog for specified file or directory.
    * By default a difference of local copy of file  with latest version stored
    * in repository is shown. The implementation of VcsProvider might choose to
    * show a dialog to allow the user for selection of  other options for diff.
    * A session must be started prior to call this method.
    * @param fileName name of file in the current working directory
    * @param ignoreCase if true, perform compare ignoring case
    * @param ignoreSpace if true, perform compare ignoring whitespace
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execShowFileDiff(String fileName, boolean ignoreCase, boolean ignoreSpace) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execShowFileDiff called");
      }
      try
      {
         JDialog myDialog = IdeWindowManagerAccess.getWindowManager().createDialog(null);
         String filePath = getCurrentWorkingDirectory() + fs + fileName;
         DiffOptionsDialog diffOptionsDialog = new DiffOptionsDialog (myDialog, true, getEnv(), filePath);
         Runner myRunner = new Runner(diffOptionsDialog);
         SwingUtilities.invokeLater(myRunner);
      }
      catch (Exception e)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execShowFileDiff Exception " + e);
         return EXEC_RESULT_ERROR;
      }
      return EXEC_RESULT_OK;
   }

   /**
    * Returns the raw result of diff command run on a file. The data returned
    * will then be parsed and displayed in IDE owned window. A session must be started prior to calling this method.
    * @param fileName name of file in the current working directory
    * @param ignoreCase if true, perform compare ignoring case
    * @param ignoreSpace if true, perform compare ignoring whitespace
    * @param diff a StringBuffer instance to return diff data in
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execGetRawFileDiff(String fileName, boolean ignoreCase, boolean ignoreSpace, StringBuffer diff)
       throws VcsProviderException
       {
          if (debugOn)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execGetRawFileDiff called");
          }
          throw new CapabilityException(CMD_GET_RAW_FILE_DIFF);
   }

   /**
    * Returns the raw result of diff command for two versions of the file. The data returned
    * will then be parsed and displayed in IDE owned window. A session must be started prior to call this method.
    * @param versionID1 identifier of one version to be compared
    * @param versionID2 identifier of another version to be compared
    * @param ignoreCase if true, perform compare ignoring case
    * @param buf a StringBuffer instance to return diff data in
    * @stereotype new
    */
   public int execGetRawVersionsDiff(String versionID1, String versionID2, boolean ignoreCase, boolean ignoreSpace, StringBuffer buf)
       throws VcsProviderException
       {
          if (debugOn)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execGetRawVersionsDiff called");
          }
          throw new CapabilityException(CMD_GET_RAW_VERSIONS_DIFF);
   }

   /**
    * Find the first version in which specified string appears A session must be started prior to call this method.
    * @param fileName - name of file in the current directory
    * @param versionId - identifier of version iof the file from which line is taken
    * @param lineNo - line number to be searched in the specified version of the file
    * @return information about the version where the specified line first appeared
    * @stereotype new
    */
   public VcsVersionInfo execFindVersionChangeAuthor(String fileName, String versionId, int lineNo) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execFindVersionChangeAuthor called");
      }
      throw new CapabilityException(CMD_FIND_CHANGE_AUTHOR);
   }

   /**
    * Shows dialog with advanced options for <CODE>command</CODE> to the user.
    * A session must be started prior to call this method.
    * @param command command to show advanced options for
    * @param project if true, the options expected to be project-specific, rather than file-specific
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execShowAdvancedOptions(String command, boolean project) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execShowAdvancedOptions called");
      }
      if (command.equals(CMD_CHECKOUT_FILES))
      {
         return openAdvancedOptions(true, false);
      }
      if (command.equals(CMD_CHECKIN_FILES))
      {
         return openAdvancedOptions(false, true);
      }
      if (command.equals(CMD_ADD_FILES))
      {
         return openAdvancedOptions(false, false);
      }
      if (command.equals(CMD_GET_FILES))
      {
         try
         {
            JDialog myDialog = IdeWindowManagerAccess.getWindowManager().createDialog(null);
            AdvancedGetOptionsDialog getDialog = new AdvancedGetOptionsDialog(myDialog, true);
            getDialog.show();
            if (getDialog.isOK())
            {
               force = getDialog.isForce();
               if (getDialog.isRev())
                  revision = getDialog.getRev();
               if (getDialog.isOther())
                  other = getDialog.getOther();
            }
         }
         catch (Exception e)
         {
            messagepage.printMessage(IdeMessageType.ERROR,
                "<Perforce Provider>: execShowAdvancedOptions failed with exception\n" + e.toString());
            return EXEC_RESULT_ERROR;
         }
         return EXEC_RESULT_OK;
      }
      return EXEC_RESULT_OK;
   }

   private int openAdvancedOptions(boolean checkOut, boolean checkIn)
   {
      try
      {
         JDialog myDialog = IdeWindowManagerAccess.getWindowManager().createDialog(null);
         Change[] changes = getChanges();
         AdvancedOptionsDialog optionsDialog = new AdvancedOptionsDialog(myDialog, true, changes, checkOut, checkIn);
         optionsDialog.show();
         if (optionsDialog.isOK())
         {
            autoSubmit = optionsDialog.isAutoSubmit();
            changeList = optionsDialog.getChangeNumber();
            changeDescription = optionsDialog.getChangeDescription();
            if (checkOut)
            {
               openForDelete = optionsDialog.isOpenForDelete();
            }
         }
      }
      catch (Exception e)
      {
         messagepage.printMessage(IdeMessageType.ERROR,
             "<Perforce Provider>: execShowAdvancedOptions:openAdvancedOptions failed with exception\n" + e.toString());
         return EXEC_RESULT_ERROR;
      }
      return EXEC_RESULT_OK;
   }

   /**
    * Clear advanced. A session must be started prior to call this method.
    * @param command command to clear advanced options for
    * @since Together 5.0
    */
   public void clearAdvancedOptions(String command) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: clearAdvancedOptions called");
      }
      if (command.equals(CMD_GET_FILES))
      {
         force = false;
         revision = "";
         other = "";
         return;
      }
      if (command.equals(CMD_ADD_FILES))
      {
         autoSubmit = true;
         changeList = 0;
         changeDescription = "";
         return;
      }
      if (command.equals(CMD_CHECKIN_FILES))
      {
         autoSubmit = true;
         changeList = 0;
         changeDescription = "";
         return;
      }
      if (command.equals(CMD_CHECKOUT_FILES))
      {
         autoSubmit = true;
         changeList = 0;
         changeDescription = "";
         openForDelete = false;
         return;
      }
   }

   /**
    * Applies a label to latest version of specified files or project in repository.
    * A session must be started prior to call this method.
    * @param files array of file names in current directory to execute command on
    * @param label label to apply
    * @param wholeProject if true, the command should be performed on the project
    * rather than on files, parameter <CODE>files</CODE> may be ignored
    * @param projectRecursively if true operation on project should be performed recursively
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execSetFilesLabel(String[] files, String label, boolean wholeProject, boolean projectRecursively)
       throws VcsProviderException
       {
          if (debugOn)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execSetFilesLabel called");
          }
          throw new CapabilityException(CMD_SET_FILES_LABEL);
   }

   /**
    * Remove the label from the files A session must be started prior to call this method.
    * @param files array of file names in current directory to execute command on
    * @param label label to be removed
    * @param wholeProject if true, the command should be performed on the project
    * rather than on files, parameter <CODE>files</CODE> may be ignored
    * @param projectRecursively if true operation on project should be performed recursively
    * @return result of command execution from EXEC_RESULT_*
    * @stereotype new
    */
   public int execUnSetFilesLabel(String[] files, String label, boolean wholeProject, boolean projectRecursively)
       throws VcsProviderException
       {
          if (debugOn)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execUnSetFilesLabel called");
          }
          throw new CapabilityException(CMD_UNSET_FILES_LABEL);
   }

   /**
    * Locks file(s) or whole project in repository, so that no one is allowed
    * to edit files but the current user. A session must be started prior to call this method.
    * @param files array of file names in current directory to execute command on
    * @param wholeProject if true, the command should be performed on the project
    * rather than on files, parameter <CODE>files</CODE> may be ignored
    * @param projectRecursively if true operation on project should be performed recursively
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execLockFiles(String[] files, boolean wholeProject, boolean projectRecursively) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execLockFiles called");
      }
      //do directory
      if (wholeProject)
      {
         String dir = "";
         try
         {
            dir = FileEntry.convertToDepotPath(getEnv(), getCurrentWorkingDirectory().getAbsolutePath());
            if (projectRecursively)
               dir = dir + "/...";
            else
               dir = dir + "/*";
            FileEntry.obtainLock(getEnv(), dir);
         }
         catch (Exception e)
         {
            messagepage.printMessage(IdeMessageType.ERROR,
                "<Perforce Provider>: execLockFiles failed with exception\n" + e.toString());
            return EXEC_RESULT_ERROR;
         }
         return EXEC_RESULT_OK;
      }
      try
      {
         for (int i = 0; i < files.length; i++)
         {
            FileEntry fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i]);
            fent.obtainLock();
            //refresh cache
            fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i], false);
         }
      }
      catch (Exception e)
      {
         messagepage.printMessage(IdeMessageType.ERROR,
             "<Perforce Provider>: execLockFiles failed with exception\n" + e.toString());
         return EXEC_RESULT_ERROR;
      }
      return EXEC_RESULT_OK;
   }

   /**
    * Unlock file(s) or the whole project. A session must be started prior to call this method.
    * @param files array of file names in current directory to execute command on
    * @param wholeProject if true, the command should be performed on the
    * project rather than on files, parameter <CODE>files</CODE> may be ignored
    * @param projectRecursively if true operation on project should be performed recursively
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execUnLockFiles(String[] files, boolean wholeProject, boolean projectRecursively) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execUnLockFiles called");
      }
      //do directory
      if (wholeProject)
      {
         String dir = "";
         try
         {
            dir = FileEntry.convertToDepotPath(getEnv(), getCurrentWorkingDirectory().getAbsolutePath());
            if (projectRecursively)
               dir = dir + "/...";
            else
               dir = dir + "/*";
            FileEntry.releaseLock(getEnv(), dir);
         }
         catch (Exception e)
         {
            messagepage.printMessage(IdeMessageType.ERROR,
                "<Perforce Provider>: execUnLockFiles failed with exception\n" + e.toString());
            return EXEC_RESULT_ERROR;
         }
         return EXEC_RESULT_OK;
      }
      try
      {
         for (int i = 0; i < files.length; i++)
         {
            FileEntry fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i]);
            fent.releaseLock();
            fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i], false);
         }
      }
      catch (Throwable e)
      {
         messagepage.printMessage(IdeMessageType.ERROR,
             "<Perforce Provider>: execUnLockFiles failed with exception\n" + e.toString());
         return EXEC_RESULT_ERROR;
      }
      return EXEC_RESULT_OK;
   }

   /**
    * Update files with latest version in the repository. Get latest version of files and delete removed files.
    * A session must be started prior to calling this method.
    * @param files array of file or directory names in working directory;
    * if 'wholeProject' is true this parameter may be null and thus should be ignored
    * @param wholeProject is true, act on all contents of the project; 'files' parameter should be ignored in this case.
    * @param projectRecursively if true, proceed the project recursively
    * @param updateInfo - information about updated files (conflicts, merges,...)
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execUpdateFiles(String[] files, boolean wholeProject, boolean projectRecursively, VcsUpdateInfo updateInfo)
       throws VcsProviderException
       {
          if (debugOn)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execUpdateFiles called");
          }
          String result = "";
          //do whole project
          if (wholeProject)
          {
             String dir = "";
             try
             {
                dir = FileEntry.convertToDepotPath(getEnv(), getCurrentWorkingDirectory().getAbsolutePath());
                if (projectRecursively)
                   dir = dir + "/...";
                else
                   dir = dir + "/*";
                String mess = FileEntry.syncWorkspace(getEnv(), dir);
                messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: " + mess);
             }
             catch (IOException e)
             {
                messagepage.printMessage(IdeMessageType.ERROR,
                    "<Perforce Provider>: execUpdateFiles failed with exception\n" + e.toString());
                return EXEC_RESULT_ERROR;
             }
             return EXEC_RESULT_OK;
          }
          //Otherwise process files
          try
          {
             for (int i = 0; i < files.length; i++)
             {
                String filePath = FileEntry.convertToDepotPath(getEnv(),
                    getCurrentWorkingDirectory().getAbsolutePath() + fs + files[i]);
                String mess = FileEntry.syncWorkspace(getEnv(), filePath);
                messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: " + mess);
                //FileEntry fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i], false);
                //fent.sync();
             }
          }
          catch (IOException ioe)
          {
             messagepage.printMessage(IdeMessageType.ERROR,
                 "<Perforce Provider>: execUpdateFiles failed with exception\n" + ioe.toString());
             return EXEC_RESULT_ERROR;
          }
          return EXEC_RESULT_OK;
   }

   public int execResolveFiles(String[] files, boolean wholeProject, boolean projectRecursively, VcsUpdateInfo updateInfo)
       throws VcsProviderException
       {
          if (debugOn)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execUpdateFiles called");
          }
          //Must do get before resolve to comply with description above of this methods behaviour
          int getResult = execGetFiles(files, wholeProject, projectRecursively, updateInfo);
          String result = "";
          //do whole project
          if (wholeProject)
          {
             String dir = "";
             try
             {
                dir = FileEntry.convertToDepotPath(getEnv(), getCurrentWorkingDirectory().getAbsolutePath());
                if (projectRecursively)
                   dir = dir + "/...";
                else
                   dir = dir + "/*";
                String mess = FileEntry.resolveAll(getEnv(), "", dir);
                //To DO: process message content
                //add flags to advanced options
                messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: " + mess);
             }
             catch (IOException e)
             {
                messagepage.printMessage(IdeMessageType.ERROR,
                    "<Perforce Provider>: execUpdateFiles failed with exception\n" + e.toString());
                return EXEC_RESULT_ERROR;
             }
             return EXEC_RESULT_OK;
          }
          //Otherwise process files
          try
          {
             for (int i = 0; i < files.length; i++)
             {
                FileEntry fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i], false);
                String mess = fent.resolve(false);
                String end = mess.substring(mess.length() - 17, mess.length() - 1);
                if ("resolve skipped.".equals(end))
                {
                   messagepage.printMessage(IdeMessageType.ERROR, "<Perforce Provider>: " + mess);
                }
                else
                {
                   messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: " + mess);
                }
             }
          }
          catch (Exception e)
          {
             messagepage.printMessage(IdeMessageType.ERROR,
                 "<Perforce Provider>: execUpdateFiles failed with exception\n" + e.toString());
             return EXEC_RESULT_ERROR;
          }
          return EXEC_RESULT_OK;
   }

   /**
    * Rename a file from current working directory in repository. A session must be started prior to call this method.
    * @param fileName a file in current working directory
    * @param newFileName new name for the file
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execRenameFile(String fileName, String newFileName) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execRenameFile called");
      }
      messagepage.printMessage(IdeMessageType.INFORMATION,
          "<Perforce Provider>: execRenameFile renaming - " + fileName + " to " + newFileName);
      fileName = getCurrentWorkingDirectory() + fs + fileName;
      newFileName = getCurrentWorkingDirectory() + fs + newFileName;
      return renameFile(fileName, newFileName);
   }

   public int renameFile(String fileName, String newFileName) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: renameFile called");
      }
      messagepage.printMessage(IdeMessageType.INFORMATION,
          "<Perforce Provider>: renameFile renaming - " + fileName + " to " + newFileName);
      Change chng = new Change(getEnv());
      try
      {
         chng.setDescription("Rename of " + fileName + " to " + newFileName);
         chng.commit();
         FileEntry fent = FileEntry.getFile(getEnv(), fileName, false);
         //get the changelist the file is open under
         String originalChange = fent.getChange();
         fent.revert(false);
         //copy new filename to somewhere
         //its writable and shouldn't be clobbered
         //any edits done prior to rename are in here
         //(as well as the class name change from the rename)
         String tempName = IdeAccess.getIdeManager().getTempFile();
         File tempFile = new File(tempName);
         File newFile = new File(newFileName);
         newFile.renameTo(tempFile);
         fent.integrate(newFileName, chng);
         fent.openForDelete(true, false, chng);
         chng.submit();
         //now move the new file content back to the
         //renamed file
         tempFile = new File(tempName);
         newFile = new File(newFileName);
         //delete the integrated file so we can write the changed one
         newFile.delete();
         tempFile.renameTo(newFile);
         tempFile.setReadOnly();
         //force refresh in cache
         FileEntry.getFile(getEnv(), newFileName, false);
         //the original file was open for edit so
         //reopen it under the same changelist
         if (originalChange != null && !originalChange.equals("default"))
         {
            Change oldChange = Change.getChange(getEnv(), originalChange, false);
            FileEntry.openForEdit(getEnv(), newFileName, false, false, false, oldChange);
         }
         else
         {
            FileEntry.openForEdit(getEnv(), newFileName, false, false, false, null);
         }
      }
      catch (CommitException ce)
      {
         messagepage.printMessage(IdeMessageType.ERROR,
             "<Perforce Provider>: execRenameFile failed with exception\n" + ce.getMessage());
         return EXEC_RESULT_ERROR;
      }
      catch (Exception e)
      {
         messagepage.printMessage(IdeMessageType.ERROR,
             "<Perforce Provider>: execRenameFile failed with exception\n" + e.toString());
         return EXEC_RESULT_ERROR;
      }
      return EXEC_RESULT_OK;
   }

   /**
    * Remove files from current working directory from repository. A session must be started prior to call this method.
    * @param comment comment to be applied to all files
    * @param files array of file names in working directory
    * @param removeLocal if true, remove local copies of files
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execRemoveFiles(String[] files, String comment, boolean removeLocal) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execRemoveFiles called");
      }

      /*    if (comment.startsWith("Renamed to") && files.length == 1) {
      fromFileName = getCurrentWorkingDirectory()+fs+files[0];
      toShort = comment.substring(11);
      return 0;
      }*/

      Change chng = new Change(getEnv());
      try
      {
         chng.setDescription(comment);
         chng.commit();
         for (int i = 0; i < files.length; i++)
         {
            FileEntry fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i], false);
            fent.revert();
            fent.openForDelete(true, false, chng);
            chng.submit();
         }
      }
      catch (SubmitException se)
      {
         messagepage.printMessage(IdeMessageType.ERROR,
             "<Perforce Provider>: execRemoveFiles failed with exception\n" + se.getMessage());
         return EXEC_RESULT_ERROR;
      }
      catch (CommitException ce)
      {
         messagepage.printMessage(IdeMessageType.ERROR,
             "<Perforce Provider>: execRemoveFiles failed with exception\n" + ce.getMessage());
         return EXEC_RESULT_ERROR;
      }
      catch (Exception e)
      {
         messagepage.printMessage(IdeMessageType.ERROR,
             "<Perforce Provider>: execRemoveFiles failed with exception\n" + e.toString());
         return EXEC_RESULT_ERROR;
      }
      return EXEC_RESULT_OK;
   }

   /**
    * Add files from current working directory to the repository.
    * Parameters keepCheckedOut and removeLocal cannot be used together. A session must be started prior to call this method.
    * @param files array of file names in working directory
    * @param fileTypes array of file type constants, one for each element of 'files';
    * Each element of 'fileTypes' is a collection of bitflags from FILE_TYPE_*
    * @param comment comment to be applied to all files
    * @param makeCheckedOut if true, make all files checked out to working directory immediately after add
    * @param removeLocal if true, remove local copies of files
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execAddFiles(String[] files, int[] fileTypes, String comment, boolean makeCheckedOut, boolean removeLocal)
       throws VcsProviderException
       {
          if (debugOn)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execAddFiles called");
          }
          Change chng = getChangeList(comment);
          if (autoSubmit)
          {
             FileEntry fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[0], false);
             chng = validateChange(chng, fent, comment);
          }
          try
          {
             for (int i = 0; i < files.length; i++)
             {
                FileEntry fent = FileEntry.openForAdd(getEnv(), getCurrentWorkingDirectory() + fs + files[i], chng);
                //force refresh of file in cache
                fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i], false);
             }
          }
          catch (Exception e)
          {
             messagepage.printMessage(IdeMessageType.ERROR,
                 "<Perforce Provider>: execAddFiles failed with exception\n" + e.toString());
             return EXEC_RESULT_ERROR;
          }
          if (!autoSubmit)
          {
             return EXEC_RESULT_OK;
          }
          //Submit the changelist
          try
          {
             chng.submit();
             if (makeCheckedOut)
             {
                execCheckOutFiles(files, comment, false, false, null);
             }
          }
          catch (SubmitException se)
          {
             messagepage.printMessage(IdeMessageType.ERROR,
                 "<Perforce Provider>: execAddFiles failed with exception\n" + se.getMessage());
             return EXEC_RESULT_ERROR;
          }
          //force refresh of files in cache
          for (int i = 0; i < files.length; i++)
          {
             //force refresh of file in cache
             FileEntry fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i], false);
          }
          return EXEC_RESULT_OK;
   }

   private Change getChangeList(String comment)
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: getChangeList called");
      }
      Change chng = null;
      //Default
      if (changeList == 0)
      {
         return chng;
      }
      //New
      if (changeList == -1)
      {
         chng = new Change(getEnv());
         try
         {
            chng.setDescription(comment);
            chng.commit();
         }
         catch (CommitException ce)
         {
            messagepage.printMessage(IdeMessageType.INFORMATION,
                "<Perforce Provider>: getChangeList failed with exception\n" + ce.getMessage());
            return null;
         }
      }
      else
      {
         chng = Change.getChange(getEnv(), changeList, false);
         if (! "".equals(changeDescription))
         {
            try
            {
               chng.setDescription(changeDescription);
               chng.commit();
            }
            catch (CommitException ce)
            {
               messagepage.printMessage(IdeMessageType.INFORMATION,
                   "<Perforce Provider>: getChangeList failed with exception\n" + ce.getMessage());
               return null;
            }
         }
      }
      return chng;
   }

   /**
    * Undo check out files. Cancel previous checkOut command. Update files from
    * repository (loose changes) and make them not checked out (readonly).
    * A session must be started prior to calling this method.
    * @param files array of file names in working directory
    * @param removeLocal if true, remove local copies of files
    * @param wholeProject is true, act on all contents of the project; 'files' parameter should be ignored in this case
    * @param projectRecursively if true, proceed the project recursively
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execUnCheckOutFiles(String[] files, boolean removeLocal, boolean wholeProject, boolean projectRecursively)
       throws VcsProviderException
       {
          if (debugOn)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execUnCheckOutFiles called");
          }
          //To Do - uncheckout whole project
          try
          {
             for (int i = 0; i < files.length; i++)
             {
                FileEntry fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i], false);
                fent.revert(removeLocal);
                //force refresh of file in cache
                fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i], false);
             }
          }
          catch (Exception e)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION,
                 "<Perforce Provider>: execUnCheckOutFiles failed with exception\n" + e.toString());
             return EXEC_RESULT_ERROR;
          }
          return EXEC_RESULT_OK;
   }

   /**
    * Check in files. Commit the changes made to files (update the repository) and make them uneditable (optionally).
    * Parameters keepCheckedOut and removeLocal cannot be used together. A session must be started prior to call this method.
    * @param files array of file or directory names in working directory; if 'wholeProject'
    * is true this parameter may be null and thus should be ignored
    * @param comment comment to be applied to all files
    * @param keepCheckedOut if true, do not perform uncheckout of files
    * @param removeLocal if true, remove local copies of files
    * @param wholeProject is true, act on all contents of the project; 'files' parameter should be ignored in this case
    * @param info information about status of check in operation
    * @param projectRecursively if true, proceed the project recursively
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execCheckInFiles(String[] files, String comment, boolean keepCheckedOut, boolean removeLocal,
       boolean wholeProject, boolean projectRecursively, VcsCommitInfo info) throws VcsProviderException
       {
          if (debugOn)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execCheckInFiles called");
          }
          //To Do - checkin of whole project
          Change chng = getChangeList(comment);
          FileEntry fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[0], false);
          chng = validateChange(chng, fent, comment);
          try
          {
             for (int i = 0; i < files.length; i++)
             {
                fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i], false);
                fent.reOpen(chng);
             }
             chng.submit();
             if (keepCheckedOut)
             {
                execCheckOutFiles(files, comment, false, false, null);
             }
          }
          catch (SubmitException se)
          {
             messagepage.printMessage(IdeMessageType.ERROR,
                 "<Perforce Provider>: execCheckInFiles SubmitException " + se.getMessage());
             return EXEC_RESULT_ERROR;
          }
          catch (Exception e)
          {
             messagepage.printMessage(IdeMessageType.ERROR, "<Perforce Provider>: execCheckInFiles Exception " + e);
             return EXEC_RESULT_ERROR;
          }
          for (int i = 0; i < files.length; i++)
          {
             //force refresh of files in cache
             fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i], false);
          }
          return EXEC_RESULT_OK;
   }

   Change validateChange(Change chng, FileEntry fent, String comment)
   {
      if (null == chng)
      {
         String change = fent.getChange();
         if ("default".equals(change))
         {
            try
            {
               chng = new Change(getEnv());
               chng.setDescription(comment);
               chng.commit();
            }
            catch (CommitException ce)
            {
               messagepage.printMessage(IdeMessageType.INFORMATION,
                   "<Perforce Provider>: getChangeList failed with exception\n" + ce.getMessage());
            }
         }
         else
         {
            chng = Change.getChange(getEnv(), change, true);
            chng.sync();
         }
      }
      return chng;
   }

   /**
    * Check out files. Make the specified files available for editing, that is
    * set them writable. A session must be started prior to call this method.
    * @param files array of file or directory names in working directory;
    * if 'wholeProject' is true this parameter may be null and thus should be ignored
    * @param comment comment to be applied to all files
    * @param wholeProject is true, act on all contents of the project; 'files' parameter should be ignored in this case
    * @param projectRecursively if true, proceed the project recursively
    * @param updateInfo - information about updated files (conflicts, merges,...)
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execCheckOutFiles(String[] files, String comment, boolean wholeProject, boolean projectRecursively, VcsUpdateInfo updateInfo)
       throws VcsProviderException
       {
          if (debugOn)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execCheckOutFiles called");
          }
          Change chng = getChangeList(comment);
          if (openForDelete && autoSubmit)
          {
             FileEntry fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[0], false);
             chng = validateChange(chng, fent, comment);
          }
          //To Do: handle checkout of whole project
          try
          {
             for (int i = 0; i < files.length; i++)
             {
                FileEntry fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i], false);
                if (openForDelete)
                {
                   fent.openForDelete(true, false, chng);
                }
                else
                {
                   fent.openForEdit(true, false, chng);
                }
                if (openForDelete && autoSubmit)
                {
                   //Submit the changelist
                   try
                   {

                      chng.submit();
                   }
                   catch (SubmitException se)
                   {
                      messagepage.printMessage(IdeMessageType.ERROR,
                          "<Perforce Provider>: execCheckOutFiles SubmitException " + se.getMessage());
                      return EXEC_RESULT_ERROR;
                   }
                }
                //force refresh of file in cache
                fent = FileEntry.getFile(getEnv(), getCurrentWorkingDirectory() + fs + files[i], false);
                if (fent.getOther() > 0)
                {
                   Vector others = fent.getOthers();
                   for (int i2 = 0; i2 < others.size(); i2++)
                   {
                      String[] other = (String[]) others.get(i);
                      if (other != null)
                      {
                         messagepage.printMessage(IdeMessageType.WARNING,
                             "<Perforce Provider>: " + fent.getName() + " - also opened by " + other[0] + " for " + other[1]);
                      }
                   }
                }
             }
          }
          catch (Exception e)
          {
             messagepage.printMessage(IdeMessageType.ERROR, "<Perforce Provider>: execCheckOutFiles Exception " + e);
             return EXEC_RESULT_ERROR;
          }
          return EXEC_RESULT_OK;
   }

   /**
    * Retrieve files from repository to working directory. A session must be started prior to call this method.
    * @param files array of file or directory names in working directory; if 'wholeProject'
    * is true this parameter may be null and thus should be ignored
    * @param wholeProject is true, act on all contents of the project; 'files' parameter should be ignored in this case
    * @param projectRecursively if true, proceed the project recursively
    * @param updateInfo - information about updated files (conflicts, merges,...)
    * @return result of command execution from EXEC_RESULT_*
    */
   public int execGetFiles(String[] files, boolean wholeProject, boolean projectRecursively, VcsUpdateInfo updateInfo)
       throws VcsProviderException
       {
          if (debugOn)
          {
             messagepage.printMessage(IdeMessageType.INFORMATION,
                 "<Perforce Provider>: execGetFiles called " + wholeProject + " - " + projectRecursively);
          }
          String result = "";
          //do Project
          if (wholeProject)
          {
             String dir = "";
             try
             {
                dir = FileEntry.convertToDepotPath(getEnv(), getCurrentWorkingDirectory().getAbsolutePath());
                if (projectRecursively)
                   dir = dir + "/...";
                else
                   dir = dir + "/*";
                if (! "".equals(revision))
                   dir = dir + "#" + revision;
                if ("".equals(revision) && ! "".equals(other))
                   dir = dir + "@" + other;
                FileEntry.syncWorkspace(getEnv(), dir, force);
             }
             catch (IOException ioe)
             {
                messagepage.printMessage(IdeMessageType.ERROR, "<Perforce Provider>: " + ioe.getMessage());
                return EXEC_RESULT_ERROR;
             }
             messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: " + result);
             return EXEC_RESULT_OK;
          }
          //do individual files
          for (int i = 0; i < files.length; i++)
          {
             String fileName = getCurrentWorkingDirectory() + fs + files[i];
             if (! "".equals(revision))
                fileName = fileName + "#" + revision;
             if ("".equals(revision) && ! "".equals(other))
                fileName = fileName + "@" + other;
             try
             {
                result = FileEntry.syncWorkspace(getEnv(), fileName, force);
             }
             catch (IOException ioe)
             {
                //if one file isn't there it shouldn't affect others
                messagepage.printMessage(IdeMessageType.ERROR, "<Perforce Provider>: " + ioe.getMessage());
             }
          }
          messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: " + result);
          return EXEC_RESULT_OK;
   }

   /**
    * Retrieve specified version of the file
    * @param versionID - provider specific version identifier
    * @return result of command execution from EXEC_RESULT_*
    * @param stream output stream to which version content will be fetched
    * @stereotype new
    */
   public int execGetVersion(String versionID, OutputStream stream) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execGetVersion called");
      }
      throw new CapabilityException(CMD_GET_VERSION);
   }

   /**
    * Checks whether file or directory is internal file used by version control system
    * @param fileName   name of file or directory
    * @return   <code>true</code> if file or directory is internal resource
    * used by version control system to maintain information about the project
    * @stereotype new
    */
   public boolean isVcsInternalFile(String fileName)
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: isVcsInternalFile called");
      }
      return false;
   }

   /**
    * Change file type
    * @param fileName a file in current working directory
    * @param newFileType one of VcsProviderConstants.FILE_TYPE_*
    * @return result of command execution from EXEC_RESULT_*
    * @stereotype new
    */
   public int execChangeFileType(String fileName, int newFileType) throws VcsProviderException
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: execChangeFileType called");
      }
      throw new CapabilityException(CMD_CHANGE_FILE_TYPE);
   }

   /**
    * Adds provider event listenter
    * @stereotype new
    */
   public void addListener(VcsProviderListener listener)
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: addListener called");
      }
   }

   /**
    * Removes provider event listenter
    * @stereotype new
    */
   public void removeListener(VcsProviderListener listener)
   {
      if (debugOn)
      {
         messagepage.printMessage(IdeMessageType.INFORMATION, "<Perforce Provider>: removeListener called");
      }
   }

   /**
    * Returns the <code>Env</code> object instance.
    * @return <code>Env</code>
    */
   private Env getEnv()
   {
      return p4Env;
   }

   public String getP4UserEmail()
   {
      return p4UserEmail;
   }

   public void setP4UserEmail(String email)
   {
      p4UserEmail = email;
   }

   private Change[] getChanges() throws PerforceException
   {
      Change[] pendingChanges = Change.getChanges(getEnv(), "", 100, null, null, false, getEnv().getUser(), "pending");
      Change[] theChanges = new Change[pendingChanges.length + 2];
      Change defaultChange = new Change(0);
      defaultChange.setDescription("Default changelist");
      theChanges[0] = defaultChange;
      Change newChange = new Change(-1);
      newChange.setDescription("New changelist");
      theChanges[1] = newChange;
      for (int i = 0; i < pendingChanges.length; i++)
      {
         theChanges[i + 2] = pendingChanges[i];
      }
      return theChanges;
   }

   private String p4UserEmail;
   private boolean debugOn = false;

   /*private String fromFileName = "";
   private String toFileName = "";
   private String fromShort = "";
   private String toShort = "";*/

   private String fs = System.getProperty("file.separator");
   private IdeMessagePage messagepage;
   private Hashtable fentCache;
   private Env p4Env;
   private String myHost;
   //Global Options
   boolean force = false;
   String revision = "";
   String other = "";
   boolean autoSubmit = true;
   boolean openForDelete = false;
   int changeList = 0;
   String changeDescription = "";

   public class Runner implements Runnable
   {
      public JDialog dlg;

      public Runner(JDialog dlg)
      {
         this.dlg = dlg;
      }

      public void run()
      {
         dlg.show();
      }
   }
}
